const cheerio = require('cheerio');
const Util = require('./util');
const puppeteer = require('puppeteer');



function datesAreValid(checkin, checkout) {
    return Util.dateDiffInDays(parseDateBestWestern(checkin), parseDateBestWestern(checkout)) >= 1;
}

async function navigateWebsite(url, checkin, checkout, adultos, criancas, bebes) {
    //let t1 = new Date();

    if (datesAreValid(checkin, checkout)) {

        const browser = await puppeteer.launch({headless: true, args: ['--no-sandbox', '--disable-setuid-sandbox']});
        try {

            const mainTab = await browser.newPage();

            // Document, Stylesheet, Image, Media, Font, Script, TextTrack, XHR, Fetch, EventSource, WebSocket, Manifest, Other.
            await mainTab.setRequestInterception(true);
            mainTab.on('request', request => {
                if (request.resourceType() === 'image' || request.resourceType() === 'font' || request.resourceType() === 'stylesheet' ) {
                    request.abort();
                }
                else {
                    request.continue();
                }

            });

            let urlBuild = `https://www.bestwestern.com.br/booking-path/hotel-rooms/${url}`;

            console.log(urlBuild);

            await mainTab.setCookie({
                name: 'search_checkInDate',
                value: checkin,
                domain: 'www.bestwestern.com.br',
                secure: true
            });
            await mainTab.setCookie({
                name: 'search_checkOutDate',
                value: checkout,
                domain: 'www.bestwestern.com.br',
                secure: true
            });

            await mainTab.setCookie({
                name: 'search_numAdults',
                value: adultos + '',
                domain: 'www.bestwestern.com.br',
                secure: true
            });
            await mainTab.setCookie({
                name: 'search_numChild',
                value: criancas + '',
                domain: 'www.bestwestern.com.br',
                secure: true
            });

            await mainTab.goto(urlBuild, {timeout: 20000});
            //console.log("Tempo Url: " + ((new Date() - t1) / 1000) + "s");

            await mainTab.waitForSelector('body');
            //console.log("Tempo body: " + ((new Date() - t1) / 1000) + "s");

            const htmlTag = await mainTab.evaluate(function (selector) {
                const selectorHtml = document.querySelector(selector);
                return selectorHtml.innerHTML
            }, 'body'); // returns innerHTML of first matching selector for class "main"


            let $ = cheerio.load(htmlTag);
            let promise = null;

            let rooms = [];


            $("div[class='group-room resultblock-room-1']").each((index, item) => {

                let room = new HotelRoom();

                let tdx = $(item);

                let exists = true;

                if (exists) {

                    room.name = tdx.find('.roomDetailsTitle').text().trim();
                    room.description = tdx.find('.roomDetailsIcons > span').text();
                    room.amountAdults = 0;
                    room.amountKids = 0;
                    let price = $(tdx.find('div[class="rateBox  initRateBox"]')[0]).attr("data-rate-amount");
                    room.price = Util.toReal(price);
                    room.total = Util.toReal(Util.dateDiffInDays(parseDateBestWestern(checkin), parseDateBestWestern(checkout)) * price);


                    tdx.find('.amenitiesListItem').each((index, itemAmenite) => {
                        room.amenites.push($(itemAmenite).text().trim());
                    });


                    tdx.find('div[class="uk-cover-background uk-position-cover"]').each((index, itemPhoto) => {
                        let photo = $(itemPhoto).attr('style').split("url(\"")[1].replace('");', '');
                        room.images.push(photo);
                    });

                    room.mainPhoto = room.images[0];


                    console.log("td", room);

                    rooms.push(room)
                }
            });


            if (rooms && rooms.length > 0) {

                let obj = {
                    data: rooms,
                    url: urlBuild,
                };

                promise = Promise.resolve(obj)
            } else {
                let obj = {
                    message: $('.error.resortRestriction').text()
                };

                promise = await Promise.reject(obj)
            }

           // console.log("Tempo: " + ((new Date() - t1) / 1000) + "s");
            await browser.close();


            return promise;

        } catch (err) {
            browser.close();
            throw err;
        }
    } else {
        let obj = {
            message: "Datas de pesquisa inválidas. Tente novamente."
        };

        return await Promise.reject(obj)
    }
}

function parseDateBestWestern(dateParam) {
    let dateSplit = dateParam.split("-");
    return new Date(dateSplit[0], dateSplit[1], dateSplit[2])
}



// navigateWebsite('best-western-premier-maceio-maceio-77119', '2018-03-19', '2018-03-21', 2, 1).then(data => {
//     console.log("sucesso")
// }).catch(error => {
//     console.log("caiu aqui");
//     console.log(error.message);
// });

class HotelRoom {

    constructor() {
        this.amenites = new Array();
        this.images = new Array();
    }

}


module.exports = {search: navigateWebsite};



